#include "multigraph.h"



multigraph::multigraph()
{

    this->n=MAXG;
    this->multig=makeDinMat(this->n);
    if(this->multig==NULL)
        printf("Error en la creación de matrices");
}
multigraph::multigraph(int numVertex)
{
    this->n=numVertex;
    this->multig=makeDinMat(this->n);
    if(this->multig==NULL)
        printf("Error en la creación de matrices");

}
void multigraph::setEdge(int i,int j){
    if (i!=j){
        this->multig[i][j]++;
        this->multig[j][i]++;
    }
    else{
        this->multig[i][j]++;
    }
}
void multigraph::getAdjacent(int vertex,setGraph * result){

    int i;
    for (i=0;i<this->n;i++)
        if(this->multig[vertex][i]>0)
            result->add(i);

}
void multigraph::removeEdge(int i,int j){
    if (this->multig[i][j]>0){
        if(i!=j){
            this->multig[i][j]--;
            this->multig[j][i]--;
        }
        else{
            this->multig[i][j]--;
        }
    }
}
void multigraph::collapseEdge(int vertex1, int vertex2){
    multigraph * auxG =new multigraph(this->n-1);
    int index [this->n];
    int i,j,newIndex;
    newIndex = 0;
    for (i = 0;i<this->n;i++){
        if(i!=vertex1 && i!= vertex2){
            index[i]=newIndex;
            newIndex ++;
        }
    }
    index[vertex2]=this->n-2;
    index[vertex1]=this->n-2;
    for (i=0;i<this->n;i++){
        for (j=0;j<this->n;j++){
            auxG->multig[index[i]][index[j]]= auxG->multig[index[i]][index[j]] + this->multig[i][j];

        }
    }

    this->copyGraph(auxG);
    if(this->multig[this->n-1][this->n-1]%2!=0)
        this->multig[this->n-1][this->n-1] = this->multig[this->n-1][this->n-1] - 2;
    else
        this->multig[this->n-1][this->n-1] = this->multig[this->n-1][this->n-1] - ((this->multig[this->n-1][this->n-1]/2)+1);
    destroyMat(auxG->multig,auxG->n);
    delete auxG;
}
void multigraph::printGraph(){
    int i,j,k;
    for (i = 0;i < this->n; i++){
        printf("Vértice %d : ",i);
        for (j = i; j<this->n;j++){
            if (this->multig[i][j] > 0){
                k=this->multig[i][j];
            }
            else{
                k=0;
            }
            while(k>0){
                printf("Vértice %d ",j);
                k--;
            }
        }
        printf("\n");
    }
    printf("\n");
}
bool multigraph::hasLoop(){
    bool result=false;
    int i;
    for (i=0;i<this->n && !result;i++){
        if(this->multig[i][i] > 0)
            result = result || true;
    }
    return result;
}

void multigraph::pathGraph(int vertex,setGraph * visited){
    stackGraph * myStack = new stackGraph(this->getNumVertex());
    int vertexAux1, i;


    myStack->push(vertex);
    visited->add(vertex);
    while(!myStack->isEmpty()){
        vertexAux1 = myStack->pop();
        setGraph * adjacents = new setGraph(this->n);
        this->getAdjacent(vertexAux1,adjacents);
        i=0;
        while(i<adjacents->getNumberOfElements()){
            if(!visited->isInSet(adjacents->get(i))){
                myStack->push(adjacents->get(i));
                visited->add(adjacents->get(i));
            }
            i++;
        }
        adjacents->destroySet();
        delete adjacents;

    }
    myStack->destroyStack();
    delete myStack;

}
int multigraph::numComponents(){    
    setGraph * visited=new setGraph(this->n);

    int result=0,i;
    for (i=0;i<this->n;i++){
        if(!visited->isInSet(i)){
            result++;
            setGraph * adjacents = new setGraph(this->n);
            this->pathGraph(i,adjacents);
            visited->addSet(adjacents);
            adjacents->destroySet();
            delete adjacents;
        }
    }
    visited->destroySet();
    delete visited;
    return result;
}
bool multigraph::isBridge(int vertex1, int vertex2){
    bool result = false;
    int numCompPrev;
    int numCompPost;

    if(vertex1!=vertex2){
        numCompPrev=this->numComponents();
        this->removeEdge(vertex1,vertex2);
        numCompPost=this->numComponents();
        if(numCompPrev<numCompPost)
            result=true;
        this->setEdge(vertex1,vertex2);
    }

    return result;
}
bool multigraph::hasBridge(){
    bool result = false;
    int i, j;
    for(i=0;i<this->n && !result;i++){
        for(j=0;j<this->n && !result;j++){
            if(i!=j && this->multig[i][j] > 0){
                result = result || this->isBridge(i,j);
            }
        }
    }
    return result;
}
int multigraph::getFirstLoop(){
    int i,result=-1;
    bool found = false;
    for(i=0;i<this->n && !found;i++){
        if(this->multig[i][i]>0){
            result = i;
            found = true;
        }
    }
    return result;
}
void multigraph::getFirstBridge(setGraph * result){

    bool found = false;
    int i, j;
    for(i=0;i<this->n && !found;i++){
        for(j=0;j<this->n && !found;j++){
            if(i!=j && this->multig[i][j] > 0){
                if(this->isBridge(i,j)){
                    result->add(i);
                    result->add(j);
                    found = true;
                }
            }
        }
    }

}
int multigraph::getNumVertex(){
    return this->n;
}
int multigraph::getNumEdges(){
    int i,j,result;
    result = 0;
    for (i=0;i<this->n;i++){
        for (j=0;j<this->n;j++){
            result = result + this->multig[i][j];
        }
    }
    return result;
}
bool multigraph::hasEdges(){
    bool result=false;
    if(this->getNumEdges()>0)
        result = true;
    return result;
}
void multigraph::copyGraph(multigraph * g){
    int i,j;
    destroyMat(this->multig,this->n);
    this->n=g->getNumVertex();

    this->multig=makeDinMat(this->n);
    for(i=0;i<this->n;i++){
        for(j=0;j<this->n;j++){
            this->multig[i][j]=g->getEdgeValue(i,j);
        }
    }
}
int multigraph::getEdgeValue(int vertex1,int vertex2){
    return this->multig[vertex1][vertex2];
}
void multigraph::setSingleEdge(int vertex1, int vertex2){
    this->multig[vertex1][vertex2]++;
}

void multigraph::getFirstEdge(setGraph * result){
    bool found = false;
    int i, j;
    for(i=0;i<this->n && !found;i++){
        for(j=0;j<this->n && !found;j++){
            if(i!=j && this->multig[i][j] > 0){
                if(!this->isBridge(i,j)){
                    result->add(i);
                    result->add(j);
                    found = true;
                }
            }
        }
    }

}
void multigraph::setNumVertex(int i){
    this->n=i;
}
void multigraph::cleanGraph(){
    int i,j;

    for (i=0;i<this->n;i++)
        for (j=0;j<this->n;j++)
            this->multig[i][j]=0;
}
void multigraph::printGraphAsAdjListInFile(FILE* f){
    int i,j,k;
    for (i = 0;i < this->n; i++){
        fprintf(f,"Vértice %d : ",i);
        for (j = 0; j<this->n;j++){
            if (this->multig[i][j] > 0){
                k=this->multig[i][j];
            }
            else{
                k=0;
            }
            while(k>0){
                fprintf(f," %d ",j);
                k--;
            }
        }
        fprintf(f,"\n");
    }
    fprintf(f,"\n");
}
void multigraph::printGraphAsAdjMatrixInFile(FILE* f){
    int i,j;
    for (i = 0;i < this->n; i++){
        for (j = 0; j<this->n;j++){
            fprintf(f,"%d ",this->multig[i][j]);
        }
        fprintf(f,"\n");
    }
    fprintf(f,"\n");
}
void multigraph::printGraphAsUpperMatrixInFile(FILE* f){
    int i,j;
    for (i = 0;i < this->n; i++){
        for (j = i; j<this->n;j++){
            fprintf(f,"%d ",this->multig[i][j]);
        }
        fprintf(f,"\n");
    }
    fprintf(f,"\n");
}
void multigraph::destroyGraph(){
    destroyMat(this->multig,this->n);
    this->multig=NULL;
}
